/*
 * NullableNonterminals.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef NULLABLE_NONTERMINALS_H_
#define NULLABLE_NONTERMINALS_H_

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include <set>
#include <deque>
#include <algorithm>

#include <grammar/Grammar.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class NullableNonterminals {
public:
	/**
	 * Retrieve all nullable nonterminals from grammar
	 * Nullable nonterminal is such nonterminal A for which holds that A ->^* \eps
	 *
	 * Source: Melichar, algorithm 2.4, step 1
	 *
	 * @param grammar grammar
	 * @return set of nullable nonterminals from grammar
	 */
	template<class T, class SymbolType = typename grammar::SymbolTypeOfGrammar < T > >
	static ext::set<SymbolType> getNullableNonterminals(const T& grammar);
};

template<class T, class SymbolType >
ext::set<SymbolType> NullableNonterminals::getNullableNonterminals(const T& grammar) {
	ext::deque<ext::set<SymbolType>> Ni;

	Ni.push_back(ext::set<SymbolType>{ });
	int i = 1;

	while(true) {
		Ni.push_back(ext::set<SymbolType>{ });
		for(const auto& rule : grammar.getRawRules()) {
			for(const auto& rhs : rule.second) {
				if(rhs.size() == 0 || std::all_of(rhs.begin(), rhs.end(), [Ni, i](const SymbolType& symb){return Ni.at(i-1).count(symb);})) {
					Ni.at(i).insert(rule.first);
				}
			}
		}

		if(Ni.at(i) == Ni.at(i-1))
			break;

		i += 1;
	}

	return Ni.at(i);
}

} /* namespace properties */

} /* namespace grammar */

#endif /* NULLABLE_NONTERMINALS_H_ */
