/*
 * ToGrammarRightRGDerivation.cpp
 *
 *  Created on: 6. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "ToGrammarRightRGDerivation.h"

#include <set>
#include <deque>
#include <vector>
#include <hexavigesimal>

#include <common/createUnique.hpp>

#include <regexp/simplify/RegExpOptimize.h>
#include <regexp/transform/RegExpDerivation.h>
#include <regexp/properties/RegExpEpsilon.h>
#include <registration/AlgoRegistration.hpp>

namespace regexp {

namespace convert {

template<class T>
grammar::RightRG < > ToGrammarRightRGDerivation::convert(const T& regexp) {
	// 1.
	T V = regexp::simplify::RegExpOptimize::optimize(regexp);

	ext::set<T> N = { V };
	ext::deque<ext::set<T>> Ni;

	Ni.push_back(ext::set<T>());
	Ni.at(0).insert(V);

	int i = 1;

	// 2.
	while(! Ni.at(i - 1).empty()) {
		Ni.push_back(ext::set<T>()); // initialize set Q_i

		for(const auto & dregexp : Ni.at( i - 1 )) {
			for(const auto & a : regexp.getAlphabet()) {
				string::LinearString < > string(ext::vector<DefaultSymbolType>{a});
				T derived = regexp::RegExpDerivation::derivation(dregexp, string);
				derived = regexp::simplify::RegExpOptimize::optimize(derived);

				// this will also add \emptyset as a regexp (and as FA state)
				if(N.count(derived) == 0) // if this state has already been found, do not add
					Ni.at(i).insert(derived);
			}
		}

		N.insert(Ni.at(i).begin(), Ni.at(i).end());
		i += 1;
	}

	// ------------------------------------------------------------------------
	// 3.

	int nonterminalId = 0;
	ext::map<T, DefaultSymbolType> nonterminalMap;

	DefaultSymbolType ntV(nonterminalId++);
	nonterminalMap.insert(std::make_pair(V, ntV));

	grammar::RightRG < > grammar(ntV);
	grammar.setTerminalAlphabet(regexp.getAlphabet());

	for(const auto & r : N) {
		if(V == r) continue;

		DefaultSymbolType nt = common::createUnique(DefaultSymbolType(ext::toBase26(nonterminalId++)), grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet());
		nonterminalMap.insert(std::make_pair(r, nt));
		grammar.addNonterminalSymbol(nt);
	}

	for(const auto & r : N) {
		for(const auto & a : regexp.getAlphabet()) {
			string::LinearString < > string(ext::vector<DefaultSymbolType>{a});
			T derived = regexp::RegExpDerivation::derivation(r, string);
			derived = regexp::simplify::RegExpOptimize::optimize(derived);

			grammar.addRule(nonterminalMap.find(r)->second, ext::make_pair(a, nonterminalMap.find(derived)->second));

			if(regexp::properties::RegExpEpsilon::languageContainsEpsilon(derived))
				grammar.addRule(nonterminalMap.find(r)->second, a);
		}
	}

	grammar.setInitialSymbol(nonterminalMap.find(V)->second);

	if(regexp::properties::RegExpEpsilon::languageContainsEpsilon(V))
		grammar.setGeneratesEpsilon(true); // okay, because of this feature we do not have to bother with extending the grammar with new rules and nonterminals. YAY!

	return grammar;
}

auto ToGrammarRightRGDerivationUnboundedRegExp = registration::AbstractRegister < ToGrammarRightRGDerivation, grammar::RightRG < >, const regexp::UnboundedRegExp < > & > ( ToGrammarRightRGDerivation::convert );
auto ToGrammarRightRGDerivationFormalRegExp = registration::AbstractRegister < ToGrammarRightRGDerivation, grammar::RightRG < >, const regexp::FormalRegExp < > & > ( ToGrammarRightRGDerivation::convert );

} /* namespace convert */

} /* namespace regexp */
