#include <list>
#include "re2faTest.h"

#include "regexp/convert/ToAutomatonDerivation.h"
#include "regexp/convert/ToAutomatonGlushkov.h"
#include "regexp/convert/ToAutomatonThompson.h"
#include "automaton/convert/ToRegExpAlgebraic.h"
#include "automaton/determinize/Determinize.h"
#include "automaton/simplify/Minimize.h"
#include "automaton/simplify/Normalize.h"
#include "automaton/simplify/EpsilonRemoverOutgoing.h"

#include "regexp/unbounded/UnboundedRegExp.h"

#include "automaton/FSM/NFA.h"
#include <factory/StringDataFactory.hpp>
#include <regexp/string/UnboundedRegExp.h>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( re2faTest, "regexp" );
CPPUNIT_TEST_SUITE_REGISTRATION( re2faTest );

void re2faTest::setUp() {
}

void re2faTest::tearDown() {
}

void re2faTest::testThompson() {
	std::string input = "a+a* b*";
	regexp::UnboundedRegExp < > regexp1 = alib::StringDataFactory::fromString (input);

	automaton::EpsilonNFA < > enfa1 = regexp::convert::ToAutomatonThompson::convert(regexp1);

	regexp::UnboundedRegExp < > regexp2 ( automaton::convert::ToRegExpAlgebraic::convert( enfa1) );

	automaton::EpsilonNFA < > enfa2 = regexp::convert::ToAutomatonThompson::convert(regexp2);

	automaton::MultiInitialStateNFA < > nfa1 = automaton::simplify::EpsilonRemoverOutgoing::remove(enfa1);
	automaton::MultiInitialStateNFA < > nfa2 = automaton::simplify::EpsilonRemoverOutgoing::remove(enfa2);

	automaton::DFA < DefaultSymbolType, ext::set < DefaultStateType > > dfa1 = automaton::determinize::Determinize::determinize(nfa1);
	automaton::DFA < DefaultSymbolType, ext::set < DefaultStateType > > dfa2 = automaton::determinize::Determinize::determinize(nfa2);

	automaton::DFA< DefaultSymbolType, unsigned > mdfa1 = automaton::simplify::Normalize::normalize(automaton::simplify::Minimize::minimize(dfa1));
	automaton::DFA< DefaultSymbolType, unsigned > mdfa2 = automaton::simplify::Normalize::normalize(automaton::simplify::Minimize::minimize(dfa2));

	CPPUNIT_ASSERT( mdfa1 == mdfa2);
}

void re2faTest::testGlushkov() {
	std::string input = "a+a*b*";
	regexp::UnboundedRegExp < > regexp1 = alib::StringDataFactory::fromString ( input );

	automaton::NFA < > nfa1 = regexp::convert::ToAutomatonGlushkov::convert ( regexp1 );

	regexp::UnboundedRegExp < > regexp2 = automaton::convert::ToRegExpAlgebraic::convert ( nfa1 );

	std::cout << regexp1 << std::endl;
	std::cout << regexp2 << std::endl;

	automaton::NFA < > nfa2 = regexp::convert::ToAutomatonGlushkov::convert(regexp2);

	automaton::DFA < DefaultSymbolType, ext::set < DefaultStateType > > dfa1 = automaton::determinize::Determinize::determinize(nfa1);
	automaton::DFA < DefaultSymbolType, ext::set < DefaultStateType > > dfa2 = automaton::determinize::Determinize::determinize(nfa2);

	automaton::DFA< DefaultSymbolType, unsigned > mdfa1 = automaton::simplify::Normalize::normalize(automaton::simplify::Minimize::minimize(dfa1));
	automaton::DFA< DefaultSymbolType, unsigned > mdfa2 = automaton::simplify::Normalize::normalize(automaton::simplify::Minimize::minimize(dfa2));

	std::cout << mdfa1 << std::endl;
	std::cout << mdfa2 << std::endl;

	CPPUNIT_ASSERT( mdfa1 == mdfa2);
}

void re2faTest::testBrzozowski() {
	std::string input = "a+a* b*";
	regexp::UnboundedRegExp < > regexp1 = alib::StringDataFactory::fromString (input);

	automaton::DFA<> dfa1 = regexp::convert::ToAutomatonDerivation::convert(regexp1);

	regexp::UnboundedRegExp < > regexp2( automaton::convert::ToRegExpAlgebraic::convert(dfa1));

	automaton::DFA<> dfa2 = regexp::convert::ToAutomatonDerivation::convert(regexp2);

	automaton::DFA<> mdfa1_2 = automaton::simplify::Minimize::minimize(dfa1);
	automaton::DFA < DefaultSymbolType, unsigned > mdfa1_3 = automaton::simplify::Normalize::normalize(mdfa1_2);

	automaton::DFA<> mdfa2_2 = automaton::simplify::Minimize::minimize(dfa2);
	automaton::DFA < DefaultSymbolType, unsigned > mdfa2_3 = automaton::simplify::Normalize::normalize(mdfa2_2);

	CPPUNIT_ASSERT( mdfa1_3 == mdfa2_3);
}
