#include "RegExpConcatenateTest.h"

#include "regexp/transform/RegExpConcatenate.h"

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExpElements.h>

#include <factory/StringDataFactory.hpp>
#include <regexp/string/UnboundedRegExp.h>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( RegExpConcatenateTest, "regexp" );
CPPUNIT_TEST_SUITE_REGISTRATION( RegExpConcatenateTest );

void RegExpConcatenateTest::setUp() {
}

void RegExpConcatenateTest::tearDown() {
}

void RegExpConcatenateTest::testRegExpConcatenate() {
	{
		std::string input1 = "(#E a b)";
		regexp::UnboundedRegExp < > re1 = alib::StringDataFactory::fromString (input1);

		std::string input2 = "(#E a c)";
		regexp::UnboundedRegExp < > re2 = alib::StringDataFactory::fromString (input2);

		regexp::UnboundedRegExp < > re = regexp::RegExpConcatenate::concatenate(re1, re2);

		std::string inputr = "(#E a b)(#E a c)";
		regexp::UnboundedRegExp < > rer = alib::StringDataFactory::fromString (inputr);

		CPPUNIT_ASSERT(re == rer);
	}
	{
		std::string input1 = "(#E a b)";
		regexp::UnboundedRegExp < > re1 = alib::StringDataFactory::fromString (input1);

		regexp::UnboundedRegExp < > re2(regexp::FormalRegExp < > (regexp::FormalRegExpStructure < DefaultSymbolType > (regexp::FormalRegExpConcatenation < DefaultSymbolType > (regexp::FormalRegExpEmpty < DefaultSymbolType > {}, regexp::FormalRegExpEpsilon < DefaultSymbolType > {}))));

		regexp::UnboundedRegExp < > re = regexp::RegExpConcatenate::concatenate(re1, re2);

		std::string inputr = "(#E a b)(#0 #E )";
		regexp::UnboundedRegExp < > rer = alib::StringDataFactory::fromString (inputr);

		CPPUNIT_ASSERT(re == rer);
	}
	{
		regexp::FormalRegExp < > re1 (regexp::FormalRegExpStructure < DefaultSymbolType > (regexp::FormalRegExpConcatenation < DefaultSymbolType > (regexp::FormalRegExpEmpty < DefaultSymbolType > {}, regexp::FormalRegExpEpsilon < DefaultSymbolType > {})));

		std::string input2 = "(#E a b)";
		regexp::UnboundedRegExp < > re2 = alib::StringDataFactory::fromString (input2);

		regexp::FormalRegExp < > re = regexp::RegExpConcatenate::concatenate(re1, regexp::FormalRegExp < > ( re2 ) );

		std::string inputr = "(#0 #E )(#E a b)";
		regexp::UnboundedRegExp < > tmp = alib::StringDataFactory::fromString (inputr);
		regexp::FormalRegExp < > rer = regexp::FormalRegExp < > ( tmp );

		std::cout << tmp.getAlphabet() << std::endl;
		std::cout << rer.getAlphabet() << std::endl;

		std::cout << re << std::endl;
		std::cout << rer << std::endl;
		CPPUNIT_ASSERT(re == rer);
	}
}
