/*
 * LL1ParseTable.h
 *
 *  Created on: 9. 6. 2015
 *	  Author: Jan Travnicek
 */

#ifndef LL_1_PARSE_TABLE_H_
#define LL_1_PARSE_TABLE_H_

#include <vector>
#include <variant>
#include <set>
#include <map>

#include <grammar/Grammar.h>
#include <string/Epsilon.h>

#include "First.h"
#include "Follow.h"

namespace grammar {

namespace parsing {

class LL1ParseTable {
public:
	template < class T, class SymbolType = typename grammar::SymbolTypeOfGrammar < T > >
	static ext::map < ext::pair < ext::variant < SymbolType, string::Epsilon < > >, SymbolType >, ext::set < ext::vector < SymbolType > > > parseTable ( const T & grammar );

};

template < class T, class SymbolType >
ext::map < ext::pair < ext::variant < SymbolType, string::Epsilon < > >, SymbolType >, ext::set < ext::vector < SymbolType > > > LL1ParseTable::parseTable ( const T & grammar ) {
	ext::map < ext::pair < ext::variant < SymbolType, string::Epsilon < > >, SymbolType >, ext::set < ext::vector < SymbolType > > > res;

	ext::map < ext::vector < SymbolType >, ext::set < ext::variant < SymbolType, string::Epsilon < > > > > first = First::first ( grammar );
	ext::map < SymbolType, ext::set < ext::variant < SymbolType, string::Epsilon < > > > > follow = Follow::follow ( grammar );

	for ( const std::pair < const SymbolType, ext::set < ext::vector < SymbolType > > > & transition : grammar.getRawRules ( ) ) {
		const SymbolType & lhs = transition.first;

		for ( const ext::vector < SymbolType > & rhs : transition.second ) {
			for ( const ext::variant < SymbolType, string::Epsilon < > > & firstElem : first[rhs] ) {
				if ( firstElem.template is < string::Epsilon < > > ( ) )
					continue;

				res [ ext::make_pair ( firstElem, lhs ) ].insert ( rhs );
			}

			if ( first[rhs].count ( ext::variant < SymbolType, string::Epsilon < SymbolType > >::template from < string::Epsilon < SymbolType > > ( ) ) )
				for ( const ext::variant < SymbolType, string::Epsilon < > > & followElem : follow[lhs] )
					res [ ext::make_pair ( followElem, lhs ) ].insert ( rhs );

		}
	}

	return res;
}

} /* namespace parsing */

} /* namespace grammar */

#endif /* LL_1_PARSE_TABLE_H_ */
