#include "LRParser.h"

#include "grammar/parsing/LRParser.h"
#include "grammar/parsing/SLR1ParseTable.h"

#include <label/LR0ItemsLabel.h>

#include <vector>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( LRParser, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( LRParser );

void LRParser::setUp ( ) {
}

void LRParser::tearDown ( ) {
}

grammar::CFG < > LRParser::getExpressionGrammar() {
	grammar::CFG < > expressionGrammar(
		{ E, T, F },
		{ plus, times, leftParenthesis, rightParenthesis, identifier },
		E
	);
	
	expressionGrammar.addRule ( { E }, { E, plus, T } );
	expressionGrammar.addRule ( { E }, { T } );
	expressionGrammar.addRule ( { T }, { T, times, F } );
	expressionGrammar.addRule ( { T }, { F } );
	expressionGrammar.addRule ( { F }, { leftParenthesis, E, rightParenthesis } );
	expressionGrammar.addRule ( { F }, { identifier } );
	
	return expressionGrammar;
}

void LRParser::testEndOfInputSymbol ( ) {
	grammar::CFG < > expressionGrammar = getExpressionGrammar ( );
	DefaultSymbolType endOfInput = grammar::parsing::LRParser::getEndOfInputSymbol ( expressionGrammar );

	bool correct = true;
	if ( expressionGrammar.getTerminalAlphabet ( ) . find ( endOfInput ) != expressionGrammar.getTerminalAlphabet ( ) . end ( ) ) {
		correct = false;
	}

	if ( expressionGrammar.getNonterminalAlphabet ( ) . find ( endOfInput ) != expressionGrammar.getNonterminalAlphabet ( ) . end ( ) ) {
		correct = false;
	}

	CPPUNIT_ASSERT ( correct );
}

void LRParser::testParseCorrectInput ( ) {
	grammar::CFG < > augmentedExpressionGrammar = grammar::parsing::LRParser::getAugmentedGrammar ( getExpressionGrammar ( ) );
	
	DefaultStateType initialState ( DefaultStateType ( label::LR0ItemsLabel ( {
		{
			augmentedExpressionGrammar.getInitialSymbol(),
			{
				{ 0 , { E } }
			}
		},
		{
			E,
			{
				{ 0 , { E, plus, T } },
				{ 0 , { T } }
			}
		},
		{
			T,
			{
				{ 0 , { T, times, F } },
				{ 0 , { F } }
			}
		},
		{
			F,
			{
				{ 0 , { leftParenthesis, E, rightParenthesis } },
				{ 0 , { identifier } }
			}
		}
	} ) ) );

	ext::vector < DefaultSymbolType > correctInput {
		leftParenthesis,
		identifier,
		plus,
		leftParenthesis,
		identifier,
		times,
		identifier ,
		rightParenthesis,
		rightParenthesis,
		grammar::parsing::LRParser::getEndOfInputSymbol ( grammar::parsing::LRParser::getAugmentedGrammar ( getExpressionGrammar ( ) ) )
	};

	bool parsingResult = grammar::parsing::LRParser::parse(
		grammar::parsing::SLR1ParseTable::getActionTable ( getExpressionGrammar ( ) ),
		grammar::parsing::SLR1ParseTable::getGotoTable ( getExpressionGrammar ( ) ),
		initialState,
		correctInput
	);

	CPPUNIT_ASSERT ( parsingResult == true );
}

void LRParser::testParseIncorrectInput ( ) {
	grammar::CFG < > augmentedExpressionGrammar = grammar::parsing::LRParser::getAugmentedGrammar ( getExpressionGrammar ( ) );
	
	DefaultStateType initialState ( DefaultStateType ( label::LR0ItemsLabel ( {
		{
			augmentedExpressionGrammar.getInitialSymbol(),
			{
				{ 0 , { E } }
			}
		},
		{
			E,
			{
				{ 0 , { E, plus, T } },
				{ 0 , { T } }
			}
		},
		{
			T,
			{
				{ 0 , { T, times, F } },
				{ 0 , { F } }
			}
		},
		{
			F,
			{
				{ 0 , { leftParenthesis, E, rightParenthesis } },
				{ 0 , { identifier } }
			}
		}
	} ) ) );

	ext::vector < DefaultSymbolType > incorrectInput {
		leftParenthesis,
		identifier,
		plus,
		leftParenthesis,
		identifier,
		times,
		identifier ,
		rightParenthesis,
		grammar::parsing::LRParser::getEndOfInputSymbol ( grammar::parsing::LRParser::getAugmentedGrammar ( getExpressionGrammar ( ) ) )
	};

	bool parsingResult = grammar::parsing::LRParser::parse(
		grammar::parsing::SLR1ParseTable::getActionTable ( getExpressionGrammar ( ) ),
		grammar::parsing::SLR1ParseTable::getGotoTable ( getExpressionGrammar ( ) ),
		initialState,
		incorrectInput
	);

	CPPUNIT_ASSERT ( parsingResult == false );
}
