/*
 * RegExpAccess.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "RegExpAccess.h"

#include <exception/CommonException.h>

#include <regexp/formal/FormalRegExp.h>
#include <regexp/formal/FormalRegExpElement.h>
#include <regexp/unbounded/UnboundedRegExp.h>
#include <regexp/unbounded/UnboundedRegExpElement.h>

#include <factory/XmlDataFactory.hpp>
#include <container/xml/ObjectsSet.h>
#include <container/xml/ObjectsMap.h>
#include <container/xml/ObjectsTuple.h>
#include <container/xml/ObjectsVector.h>
#include <container/xml/ObjectsVariant.h>
#include <container/xml/ObjectsPair.h>
#include <container/xml/ObjectsTree.h>

#include "common/AccessHelpers.hpp"
#include <registration/AlgoRegistration.hpp>

namespace dataAccess {

void RegExpAccess::access ( regexp::RegExp & regexp, const RegExpSettings::Settings & settings, const OperationSettings::Settings & operation, ext::deque < sax::Token > & argument ) {
	dispatch ( regexp.getData ( ), settings, operation, argument );
}

void RegExpAccess::access ( regexp::FormalRegExp < > & regexp, const RegExpSettings::Settings & settings, const OperationSettings::Settings & operation, ext::deque < sax::Token > & argument ) {
	if ( settings == RegExpSettings::Settings::ALPHABET )
		return handleComponent < regexp::GeneralAlphabet > ( regexp, operation, argument );

	if ( settings == RegExpSettings::Settings::CONTENT && operation == OperationSettings::Settings::GET ) alib::XmlDataFactory::toStdout ( regexp.getRegExp ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto RegExpAccessFormalRegExp = registration::OverloadRegister < RegExpAccess, void, regexp::FormalRegExp < > > ( RegExpAccess::access );

void RegExpAccess::access ( regexp::UnboundedRegExp < > & regexp, const RegExpSettings::Settings & settings, const OperationSettings::Settings & operation, ext::deque < sax::Token > & argument ) {
	if ( settings == RegExpSettings::Settings::ALPHABET )
		return handleComponent < regexp::GeneralAlphabet > ( regexp, operation, argument );

	if ( settings == RegExpSettings::Settings::CONTENT && operation == OperationSettings::Settings::GET ) alib::XmlDataFactory::toStdout ( regexp.getRegExp ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto RegExpAccessUnboundedRegExp = registration::OverloadRegister < RegExpAccess, void, regexp::UnboundedRegExp < > > ( RegExpAccess::access );

} /* namespace dataAccess */
