/*
 * vector.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_VECTOR_H_
#define OBJECTS_VECTOR_H_

#include <vector>
#include <string>
#include <sstream>

#include "ContainerBase.h"
#include "ContainerFeatures.h"

#include <object/UniqueObject.h>
#include <object/AnyObject.h>

#include <core/normalize.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
template < class ElementType >
class ObjectsVector final : public ext::vector < ElementType >, public ContainerBase {
public:
	explicit ObjectsVector ( ext::vector < ElementType > );
	explicit ObjectsVector ( );

	virtual ContainerBase * clone ( ) const;

	virtual ContainerBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsVector & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	virtual alib::ObjectBase * inc ( ) &&;

	typedef ObjectsVector < > normalized_type;
};

template < class ElementType >
ObjectsVector < ElementType >::ObjectsVector ( ext::vector < ElementType > raw ) : ext::vector < ElementType > ( std::move ( raw ) ) {
}

template < class ElementType >
ObjectsVector < ElementType >::ObjectsVector ( ) : ext::vector < ElementType > ( ) {
}

template < class ElementType >
ContainerBase * ObjectsVector < ElementType >::clone ( ) const {
	return new ObjectsVector ( * this );
}

template < class ElementType >
ContainerBase * ObjectsVector < ElementType >::plunder ( ) && {
	return new ObjectsVector ( std::move ( * this ) );
}

template < class ElementType >
int ObjectsVector < ElementType >::compare ( const ObjectsVector & other ) const {
	static ext::compare < ext::vector < ElementType > > comp;

	return comp ( static_cast < const ext::vector < ElementType > & > ( * this ), static_cast < const ext::vector < ElementType > & > ( other ) );
}

template < class ElementType >
void ObjectsVector < ElementType >::operator >>( std::ostream & os ) const {
	os << "(ObjectsVector " << static_cast < const ext::vector < ElementType > & > ( * this ) << ")";
}

template < class ElementType >
ObjectsVector < ElementType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return std::move ( ss ).str ( );
}

template < class ElementType >
alib::ObjectBase* ObjectsVector < ElementType >::inc() && {
	return new alib::UniqueObject(alib::Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace container */

namespace alib {

template < >
struct normalize < ext::vector < alib::Object > > {
	static ext::vector < alib::Object > && eval ( ext::vector < alib::Object > && source ) {
		return std::move ( source );
	}
};

template < class ElementType >
struct normalize < ext::vector < ElementType > > {
	static ext::vector < alib::Object > eval ( ext::vector < ElementType > && source ) {
		ext::vector < alib::Object > res;

		for ( ElementType & element : source )
			res.push_back ( alib::Object ( alib::AnyObject < ElementType > ( std::move ( element ) ) ) );

		return res;
	}
};

template < class ElementType >
struct normalize < container::ObjectsVector < ElementType >, typename std::enable_if < ! std::is_same < container::ObjectsVector < ElementType >, container::ObjectsVector < > >::value >::type > {
	static container::ObjectsVector < > eval ( container::ObjectsVector < ElementType > && value ) {
		return container::ObjectsVector < > ( normalize < ext::vector < ElementType > >::eval ( std::move ( value ) ) );
	}
};

} /* namespace alib */

#endif /* OBJECTS_VECTOR_H_ */
