/*
 * UnboundedRegExpElement.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef VISITOR_HPP_
#define VISITOR_HPP_

#include <utility>

namespace alib {

template < class ... Visitables >
class VisitorContextBaseBuilder;

template < class Visitable >
class VisitorContextBaseBuilder < Visitable > {
public:
	virtual void visit ( const Visitable & ) = 0;
};

template < class Visitable, class ... Visitables >
class VisitorContextBaseBuilder < Visitable, Visitables ... > : public VisitorContextBaseBuilder < Visitables ... > {
public:
	using VisitorContextBaseBuilder < Visitables ... >::visit;

	virtual void visit ( const Visitable & ) = 0;
};

template < class ... Visitables >
class VisitorContextBase : public VisitorContextBaseBuilder < Visitables ... > {
};

// -----------------------------------------------------------------

template < class ReturnType, class Visitor, class ... Params >
class VisitorContextAux {
	typename std::aligned_storage < ext::SizeOf < ReturnType >::size, ext::AlignOf < ReturnType >::align >::type result;

	ext::tuple < Params ... > m_params;

public:
	VisitorContextAux ( Params && ... params ) : m_params ( std::forward < Params > ( params ) ... ) {
	}

	template < class Inherit, size_t ... Indexes, class ReturnType2 = ReturnType >
	void call ( const Inherit & inherit, std::index_sequence < Indexes ... >, typename std::enable_if < !std::is_void < ReturnType2 >::value && std::is_same < ReturnType, ReturnType2 >::value >::type * = 0 ) {
		new ( & result )ReturnType ( Visitor::visit ( inherit, std::forward < Params > ( std::get < Indexes > ( m_params ) ) ... ) );
	}

	template < class Inherit, size_t ... Indexes, class ReturnType2 = ReturnType >
	void call ( const Inherit & inherit, std::index_sequence < Indexes ... >, typename std::enable_if < std::is_void < ReturnType2 >::value && std::is_same < ReturnType, ReturnType2 >::value >::type * = 0 ) {
		ReturnType ( Visitor::visit ( inherit, std::forward < Params > ( std::get < Indexes > ( m_params ) ) ... ) );
	}

	template < class ReturnType2 = ReturnType >
	ReturnType getResult ( typename std::enable_if < !std::is_void < ReturnType2 >::value && std::is_same < ReturnType, ReturnType2 >::value >::type * = 0 ) {
		ReturnType res = reinterpret_cast < ReturnType && > ( result );

		 // reinterpret_cast < ReturnType * > ( & result )->~ReturnType ( ); // not needed to delete; data gets moved from on previous line
		return res;
	}

	template < class ReturnType2 = ReturnType >
	void getResult ( typename std::enable_if < std::is_void < ReturnType2 >::value && std::is_same < ReturnType, ReturnType2 >::value >::type * = 0 ) {
	}

};

} /* namespace alib */

#endif /* VISITOR_HPP_ */
