/*
 * NonlinearVariableSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef NONLINEAR_VARIABLE_SYMBOL_H_
#define NONLINEAR_VARIABLE_SYMBOL_H_

#include "Symbol.h"
#include "SymbolFeatures.h"
#include "UniqueSymbol.h"

#include <alphabet/common/SymbolNormalize.h>

#include <core/normalize.hpp>

namespace alphabet {

/**
 * Represents subtreeWildcard symbol for tree linearization.
 */
template < class SymbolType >
class NonlinearVariableSymbol : public SymbolBase {
	SymbolType m_symbol;

public:
	explicit NonlinearVariableSymbol ( int number );
	explicit NonlinearVariableSymbol ( char character );
	explicit NonlinearVariableSymbol ( std::string symbol );
	explicit NonlinearVariableSymbol ( SymbolType symbol );

	/**
	 * Creates a subtreeWildcard symbol.
	 * @param symbol name of the symbol
	 */
	explicit NonlinearVariableSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const SymbolType & getSymbol ( ) const &;

	/**
	 * @return name of the symbol
	 */
	SymbolType && getSymbol ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const NonlinearVariableSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	virtual SymbolBase * inc ( ) &&;

	typedef NonlinearVariableSymbol < > normalized_type;
};

template < class SymbolType >
NonlinearVariableSymbol < SymbolType >::NonlinearVariableSymbol ( int number ) : m_symbol ( DefaultSymbolType ( number ) ) {
}

template < class SymbolType >
NonlinearVariableSymbol < SymbolType >::NonlinearVariableSymbol ( char character ) : m_symbol ( DefaultSymbolType ( character ) ) {
}

template < class SymbolType >
NonlinearVariableSymbol < SymbolType >::NonlinearVariableSymbol ( std::string symbol ) : m_symbol ( DefaultSymbolType ( std::move ( symbol ) ) ) {
}

template < class SymbolType >
NonlinearVariableSymbol < SymbolType >::NonlinearVariableSymbol ( SymbolType symbol ) : m_symbol ( std::move ( symbol ) ) {
}

template < class SymbolType >
SymbolBase * NonlinearVariableSymbol < SymbolType >::clone ( ) const {
	return new NonlinearVariableSymbol ( * this );
}

template < class SymbolType >
SymbolBase * NonlinearVariableSymbol < SymbolType >::plunder ( ) && {
	return new NonlinearVariableSymbol ( std::move ( * this ) );
}

template < class SymbolType >
const SymbolType & NonlinearVariableSymbol < SymbolType >::getSymbol ( ) const & {
	return m_symbol;
}

template < class SymbolType >
SymbolType && NonlinearVariableSymbol < SymbolType >::getSymbol ( ) && {
	return std::move ( m_symbol );
}

template < class SymbolType >
int NonlinearVariableSymbol < SymbolType >::compare ( const NonlinearVariableSymbol & other ) const {
	static ext::compare < decltype ( m_symbol ) > comp;

	return comp ( m_symbol, other.m_symbol );
}

template < class SymbolType >
void NonlinearVariableSymbol < SymbolType >::operator >>( std::ostream & out ) const {
	out << "(NonlinearVariableSymbol " << m_symbol << ")";
}

template < class SymbolType >
NonlinearVariableSymbol < SymbolType >::operator std::string ( ) const {
	return "$" + ext::to_string ( m_symbol );
}

template < class SymbolType >
SymbolBase * NonlinearVariableSymbol < SymbolType >::inc ( ) && {
	return new UniqueSymbol ( Symbol ( std::move ( * this ) ), primitive::Integer ( 0 ) );
}

} /* namespace alphabet */

namespace alib {

template < class SymbolType >
struct normalize < alphabet::NonlinearVariableSymbol < SymbolType >, typename std::enable_if < ! std::is_same < alphabet::NonlinearVariableSymbol < SymbolType >, alphabet::NonlinearVariableSymbol < > >::value >::type > {
	static alphabet::NonlinearVariableSymbol < > eval ( alphabet::NonlinearVariableSymbol < SymbolType > && value ) {
		return alphabet::NonlinearVariableSymbol < > ( alphabet::SymbolNormalize::normalizeSymbol ( std::move ( value ).getSymbol ( ) ) );
	}
};

} /* namespace alib */

#endif /* NONLINEAR_VARIABLE_SYMBOL_H_ */
