/*
 * StartSymbol.h
 *
 *  Created on: Jun 19, 2014
 *      Author: Jan Travnicek
 */

#ifndef START_SYMBOL_H_
#define START_SYMBOL_H_

#include "SymbolBase.h"
#include "Symbol.h"
#include <object/Object.h>

namespace alphabet {

/**
 * Represents blank symbol in an alphabet.
 */
class StartSymbol : public SymbolBase {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit StartSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const StartSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	template < typename Base >
	static inline typename std::enable_if <   std::is_integral < Base >::value, Base >::type instance ( );
	template < typename Base >
	static inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type instance ( );

	virtual SymbolBase * inc ( ) &&;

	typedef StartSymbol normalized_type;
};

template < typename Base >
inline typename std::enable_if < std::is_integral < Base >::value, Base >::type StartSymbol::instance ( ) {
	return '^';
}

template < >
inline std::string StartSymbol::instance < std::string > ( ) {
	return std::string ( 1, StartSymbol::instance < char > ( ) );
}

template < >
inline StartSymbol StartSymbol::instance < StartSymbol > ( ) {
	return StartSymbol ( );
}

template < >
inline Symbol StartSymbol::instance < Symbol > ( ) {
	return Symbol ( StartSymbol ( ) );
}

template < >
inline alib::Object StartSymbol::instance < alib::Object > ( ) {
	return alib::Object ( StartSymbol ( ) );
}

} /* namespace alphabet */

#endif /* START_SYMBOL_H_ */
