/*
 * InitialStateLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "InitialStateLabel.h"
#include <sax/FromXMLParserHelper.h>
#include "Label.h"
#include <object/Object.h>
#include "UniqueLabel.h"

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace label {

InitialStateLabel::InitialStateLabel() {

}

LabelBase* InitialStateLabel::clone() const {
	return new InitialStateLabel(*this);
}

LabelBase* InitialStateLabel::plunder() && {
	return new InitialStateLabel(std::move(*this));
}

int InitialStateLabel::compare(const InitialStateLabel&) const {
	return 0;
}

void InitialStateLabel::operator>>(std::ostream& out) const {
	out << "(InitialStateLabel)";
}

InitialStateLabel::operator std::string() const {
	return InitialStateLabel::instance < std::string > ( );
}

InitialStateLabel InitialStateLabel::parse(ext::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, InitialStateLabel::getXmlTagName());
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, InitialStateLabel::getXmlTagName());
	return InitialStateLabel();
}

void InitialStateLabel::compose(ext::deque<sax::Token>& out, const InitialStateLabel & ) {
	out.emplace_back(InitialStateLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(InitialStateLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

LabelBase* InitialStateLabel::inc() && {
	return new UniqueLabel(Label(std::move(*this)), primitive::Integer(0));
}

} /* namespace label */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < label::InitialStateLabel > ( );

static auto xmlWrite = registration::XmlWriterRegister < label::Label, label::InitialStateLabel > ( );
static auto xmlWrite2 = registration::XmlWriterRegister < alib::Object, label::InitialStateLabel > ( );

static auto xmlReader = registration::XmlReaderRegister < label::Label, label::InitialStateLabel > ( );
static auto xmlReaded2 = registration::XmlReaderRegister < alib::Object, label::InitialStateLabel > ( );

} /* namespace */
