/*
 * ObjectLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef OBJECT_LABEL_H_
#define OBJECT_LABEL_H_

#include <object/Object.h>
#include "LabelBase.h"
#include "Label.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class ObjectLabel final : public LabelBase {
protected:
	alib::Object m_label;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit ObjectLabel ( alib::Object label );

	virtual LabelBase * clone ( ) const;

	virtual LabelBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const alib::Object & getData ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectLabel & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "ObjectLabel";

		return xmlTagName;
	}

	static ObjectLabel parse ( ext::deque < sax::Token >::iterator & input );

	static void compose ( ext::deque < sax::Token > & out, const ObjectLabel & label );

	virtual LabelBase * inc ( ) &&;

	typedef ObjectLabel normalized_type;
};

} /* namespace label */

#endif /* OBJECT_LABEL_H_ */
