#ifndef _RAW_REGISTRATION_HPP_
#define _RAW_REGISTRATION_HPP_

#include <core/xmlApi.hpp>

#include <abstraction/RawReaderRegistry.hpp>
#include <abstraction/RawWriterRegistry.hpp>

#include <abstraction/AlgorithmRegistry.hpp>

namespace raw {

class Parse {
public:
	static std::shared_ptr < abstraction::OperationAbstraction > abstractionFromString ( const std::string & type, const std::string & ) {
		return abstraction::RawReaderRegistry::getAbstraction ( type );
	}

};

class Compose {
public:
	template < class Type >
	static std::shared_ptr < abstraction::OperationAbstraction > abstractionFromType ( const Type & ) {
		return abstraction::RawWriterRegistry::getAbstraction ( ext::to_string < Type > ( ) );
	}

};

} /* namespace raw */

namespace registration {

template < class Type >
class RawReaderRegister {
public:
	RawReaderRegister ( ) {
		abstraction::RawReaderRegistry::registerRawReader < Type > ( );
		abstraction::AlgorithmRegistry::registerWrapper < raw::Parse, const std::string &, const std::string & > ( raw::Parse::abstractionFromString, std::array < std::string, 2 > { } );
	}
};

template < class Type >
class RawWriterRegister {
public:
	RawWriterRegister ( ) {
		abstraction::RawWriterRegistry::registerRawWriter < Type > ( );
		abstraction::AlgorithmRegistry::registerWrapper < raw::Compose, const Type & > ( raw::Compose::abstractionFromType, std::array < std::string, 1 > { } );
	}
};

} /* namespace registration */

#endif // _RAW_REGISTRATION_HPP_
