#include "RangeTest.h"
#include <range>
#include <vector>
#include <list>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( RangeTest, "bits" );
CPPUNIT_TEST_SUITE_REGISTRATION ( RangeTest );

void RangeTest::setUp ( ) {
}

void RangeTest::tearDown ( ) {
}

void RangeTest::constructorTest ( ) {
	int arr[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };

	ext::range < int * > ra1 ( arr, arr + sizeof ( arr ) / sizeof ( * arr ) );

	ext::vector < int > v1 = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };

	ext::range < ext::vector < int >::iterator > rv1 ( v1.begin ( ), v1.end ( ) );

	CPPUNIT_ASSERT ( ( std::equal ( ra1.begin ( ), ra1.end ( ), rv1.begin ( ), rv1.end ( ) ) ) );

	auto rv2 = ext::make_range ( v1 );

	CPPUNIT_ASSERT ( ( std::equal ( rv1.begin ( ), rv1.end ( ), rv2.begin ( ), rv2.end ( ) ) ) );
}

void RangeTest::sizeTest ( ) {
	int arr[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };

	ext::range < int * > ra1 ( arr, arr + sizeof ( arr ) / sizeof ( * arr ) );

	CPPUNIT_ASSERT ( ra1.size ( ) == 10 );

	ext::vector < int > v1 = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };

	ext::range < ext::vector < int >::iterator > rv1 ( v1.begin ( ), v1.end ( ) );

	CPPUNIT_ASSERT ( rv1.size ( ) == 10 );

	ext::list < int > l1 = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };

	ext::range < ext::list < int >::iterator > rl1 ( l1.begin ( ), l1.end ( ) );

	CPPUNIT_ASSERT ( rl1.size ( ) == 10 );
}

void RangeTest::splitTest ( ) {
	ext::vector < int > v1 = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };

	auto r1 = ext::make_range ( v1 );

	ext::vector < int > v2 = { 0, 1, 2 };

	auto r2 = ext::make_range ( v2 );

	ext::vector < int > v3 = { 3, 4, 5, 6, 7, 8, 9 };

	auto r3 = ext::make_range ( v3 );

	auto splitted = r1.split ( 3 );

	CPPUNIT_ASSERT ( ( std::equal ( splitted.first.begin ( ), splitted.first.end ( ), r2.begin ( ), r2.end ( ) ) ) );
	CPPUNIT_ASSERT ( ( std::equal ( splitted.second.begin ( ), splitted.second.end ( ), r3.begin ( ), r3.end ( ) ) ) );
}

void RangeTest::sliceTest ( ) {
	ext::vector < int > v1 = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };

	auto r1 = ext::make_range ( v1 );

	ext::vector < int > v2 = { 4, 5, 6 };

	auto r2 = ext::make_range ( v2 );

	auto sliced = r1.slice ( 4, -3 );

	CPPUNIT_ASSERT ( ( std::equal ( sliced.begin ( ), sliced.end ( ), r2.begin ( ), r2.end ( ) ) ) );
}
