/*
 * EpsilonFreeCFG.h
 *
 * Created on: Sep 27, 2017
 * Author: Jan Travnicek
 */

#ifndef _STRING_EPSILON_FREE_CFG_H_
#define _STRING_EPSILON_FREE_CFG_H_

#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <core/stringApi.hpp>

#include <grammar/GrammarFromStringLexer.h>

#include <grammar/string/common/GrammarFromStringParserCommon.h>
#include <grammar/string/common/GrammarToStringComposerCommon.h>

namespace alib {

template<class SymbolType >
struct stringApi < grammar::EpsilonFreeCFG < SymbolType > > {
	static grammar::EpsilonFreeCFG < SymbolType > parse ( std::istream & input );
	static bool first ( std::istream & input );
	static void compose ( std::ostream & output, const grammar::EpsilonFreeCFG < SymbolType > & grammar );
};

template<class SymbolType >
grammar::EpsilonFreeCFG < SymbolType > stringApi < grammar::EpsilonFreeCFG < SymbolType > >::parse ( std::istream & input ) {
	grammar::GrammarFromStringLexer::Token token = grammar::GrammarFromStringLexer::next(input);
	if(token.type != grammar::GrammarFromStringLexer::TokenType::EPSILON_FREE_CFG)
		throw exception::CommonException("Unrecognised EpsilonFreeCFG token.");

	return grammar::GrammarFromStringParserCommon::parseCFLikeGrammar < grammar::EpsilonFreeCFG < SymbolType > > ( input );
}

template<class SymbolType >
bool stringApi < grammar::EpsilonFreeCFG < SymbolType > >::first ( std::istream & input ) {
	grammar::GrammarFromStringLexer::Token token = grammar::GrammarFromStringLexer::next ( input );
	bool res = token.type == grammar::GrammarFromStringLexer::TokenType::EPSILON_FREE_CFG;
	grammar::GrammarFromStringLexer::putback ( input, token );
	return res;
}

template<class SymbolType >
void stringApi < grammar::EpsilonFreeCFG < SymbolType > >::compose ( std::ostream & output, const grammar::EpsilonFreeCFG < SymbolType > & grammar ) {
	output << "EPSILON_FREE_CFG";
	grammar::GrammarToStringComposerCommon::composeCFLikeGrammar ( output, grammar );
}

} /* namespace alib */

#endif /* _STRING_EPSILON_FREE_CFG_H_ */
