/*
 * LeftRG.h
 *
 * Created on: Sep 27, 2017
 * Author: Jan Travnicek
 */

#ifndef _STRING_LEFT_RG_H_
#define _STRING_LEFT_RG_H_

#include <grammar/Regular/LeftRG.h>
#include <core/stringApi.hpp>

#include <grammar/GrammarFromStringLexer.h>

#include <grammar/string/common/GrammarFromStringParserCommon.h>
#include <grammar/string/common/GrammarToStringComposerCommon.h>

namespace alib {

template<class SymbolType >
struct stringApi < grammar::LeftRG < SymbolType > > {
	static grammar::LeftRG < SymbolType > parse ( std::istream & input );
	static bool first ( std::istream & input );
	static void compose ( std::ostream & output, const grammar::LeftRG < SymbolType > & grammar );
};

template<class SymbolType >
grammar::LeftRG < SymbolType > stringApi < grammar::LeftRG < SymbolType > >::parse ( std::istream & input ) {
	grammar::GrammarFromStringLexer::Token token = grammar::GrammarFromStringLexer::next(input);
	if(token.type != grammar::GrammarFromStringLexer::TokenType::LEFT_RG)
		throw exception::CommonException("Unrecognised LeftRG token.");

	return grammar::GrammarFromStringParserCommon::parseCFLikeGrammar < grammar::LeftRG < SymbolType > > ( input );
}

template<class SymbolType >
bool stringApi < grammar::LeftRG < SymbolType > >::first ( std::istream & input ) {
	grammar::GrammarFromStringLexer::Token token = grammar::GrammarFromStringLexer::next ( input );
	bool res = token.type == grammar::GrammarFromStringLexer::TokenType::LEFT_RG;
	grammar::GrammarFromStringLexer::putback ( input, token );
	return res;
}

template<class SymbolType >
void stringApi < grammar::LeftRG < SymbolType > >::compose ( std::ostream & output, const grammar::LeftRG < SymbolType > & grammar ) {
	output << "LEFT_RG";
	grammar::GrammarToStringComposerCommon::composeCFLikeGrammar ( output, grammar );
}

} /* namespace alib */

#endif /* _STRING_LEFT_RG_H_ */
