#ifndef _STRING_REGISTRATION_HPP_
#define _STRING_REGISTRATION_HPP_

#include <core/xmlApi.hpp>

#include <abstraction/StringWriterRegistry.hpp>
#include <abstraction/StringReaderRegistry.hpp>

#include <abstraction/AlgorithmRegistry.hpp>

namespace string {

class Parse {
public:
	static std::shared_ptr < abstraction::OperationAbstraction > abstractionFromString ( const std::string & group, std::string && data ) {
		return abstraction::StringReaderRegistry::getAbstraction ( group, data );
	}

};

class Compose {
public:
	template < class Type >
	static std::shared_ptr < abstraction::OperationAbstraction > abstractionFromType ( const Type & ) {
		return abstraction::StringWriterRegistry::getAbstraction ( ext::to_string < Type > ( ) );
	}

};

} /* namespace string */

namespace registration {

template < class Group, class Type >
class StringReaderRegister {
public:
	StringReaderRegister ( ) {
		alib::stringApi < Group >::template registerStringReader < Type > ( );

		abstraction::StringReaderRegistry::registerStringReader < Group, Type > ( );
		abstraction::AlgorithmRegistry::registerWrapper < string::Parse, const std::string &, std::string && > ( string::Parse::abstractionFromString, std::array < std::string, 2 > { } );
	}
};

template < class Group, class Type >
class StringWriterRegister {
public:
	StringWriterRegister ( ) {
		alib::stringApi < Group >::template registerStringWriter < Type > ( );

		abstraction::StringWriterRegistry::registerStringWriter < Type > ( );
		abstraction::AlgorithmRegistry::registerWrapper < string::Compose, const Type & > ( string::Compose::abstractionFromType, std::array < std::string, 1 > { } );
	}
};

} /* namespace registration */

#endif // _STRING_REGISTRATION_HPP_
