/*
 * Deque.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef _XML_OBJECTS_DEQUE_H_
#define _XML_OBJECTS_DEQUE_H_

#include <container/ObjectsDeque.h>
#include <core/xmlApi.hpp>

namespace alib {

template < typename T >
struct xmlApi < ext::deque < T > > {
	static ext::deque < T > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const ext::deque < T > & data );
};

template < typename T >
ext::deque < T > xmlApi < ext::deque < T > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );

	ext::deque < T > set;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		set.push_back ( alib::xmlApi < T >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return set;
}

template < typename T >
bool xmlApi < ext::deque < T > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

template < typename T >
const std::string & xmlApi < ext::deque < T > >::xmlTagName ( ) {
	static std::string xmlTagName = "Deque";

	return xmlTagName;
}

template < typename T >
void xmlApi < ext::deque < T > >::compose ( ext::deque < sax::Token > & output, const ext::deque < T > & input ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );

	for ( const T & item : input )
		alib::xmlApi < T >::compose ( output, item );

	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

template < typename T >
struct xmlApi < container::ObjectsDeque < T > > {
	static container::ObjectsDeque < T > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const container::ObjectsDeque < T > & data );
};

template < typename T >
container::ObjectsDeque < T > xmlApi < container::ObjectsDeque < T > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return container::ObjectsDeque < T > ( xmlApi < ext::deque < T > >::parse ( input ) );
}

template < typename T >
bool xmlApi < container::ObjectsDeque < T > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return xmlApi < ext::deque < T > >::first ( input );
}

template < typename T >
const std::string & xmlApi < container::ObjectsDeque < T > >::xmlTagName ( ) {
	return xmlApi < ext::deque < T > >::xmlTagName ( );
}

template < typename T >
void xmlApi < container::ObjectsDeque < T > >::compose ( ext::deque < sax::Token > & output, const container::ObjectsDeque < T > & input ) {
	xmlApi < ext::deque < T > >::compose ( output, input );
}

} /* namespace alib */

#endif /* _XML_OBJECTS_DEQUE_H_ */
